import re
from collections import namedtuple
from datetime import date
from pathlib import Path
from typing import Optional

import yaml
from dateutil.parser import parse
from justice_api import justices_tbl, load_justices
from sqlite_utils import Database

from .tables import create_decision_tbl, create_voting_lines_tbl

BASE_CONTENT = Path().home() / "code" / "rawlaw"
DB_FILE = BASE_CONTENT.joinpath("index.db")
DB = Database(DB_FILE, use_counts_table=True)

DECISIONS_PATH: Path = BASE_CONTENT.joinpath("decisions")
SC_PATH: Path = DECISIONS_PATH.joinpath("sc")
OLD_PATH: Path = DECISIONS_PATH.joinpath("legacy")

JUSTICES_TABLE_NAME = "Justices"
DECISIONS_TABLE_NAME = "Decisions"
VOTING_LINES_TABLE_NAME = "VotingLines"
MAX_LENGTH_IDX = 100
MAX_RAW_PONENTE = 35
MAX_CASE_TITLE = 1000
MAX_FALLO = 20000
PER_CURIAM_PATTERN = re.compile(r"per\s+curiam", re.I)
PONENCIA_HTML_FILE = "ponencia.html"
ANNEX_HTML_FILE = "annex.html"

MatchedRows = namedtuple("MatchedRows", ["counts", "rows"])


def parse_date_if_exists(text: Optional[str]) -> Optional[date]:
    if not text:
        return None
    elif text and len(text) < 5:
        return None
    elif not (parsed := parse(text)):
        return None
    return parsed.date()


def load_yaml_from_path(file_path: Path):
    with open(file_path, "r") as readfile:
        return yaml.load(readfile, Loader=yaml.FullLoader)


JUSTICES = (
    justices_tbl()
    if not DB[JUSTICES_TABLE_NAME].exists()
    else load_justices(DB[JUSTICES_TABLE_NAME])
)
if not JUSTICES.exists():
    raise Exception("Justices prerequisite to Decisions.")

DECISIONS = (
    DB[DECISIONS_TABLE_NAME]
    if DB[DECISIONS_TABLE_NAME].exists()
    else create_decision_tbl(DB, DECISIONS_TABLE_NAME)
)
if not DECISIONS.exists():
    raise Exception("Decisions prerequisite to Voting Lines.")

VOTING_LINES = (
    DB[VOTING_LINES_TABLE_NAME]
    if DB[VOTING_LINES_TABLE_NAME].exists()
    else create_voting_lines_tbl(DB, VOTING_LINES_TABLE_NAME)
)
