from datetime import date
from typing import Match, NoReturn, Optional, Union

from ..config import DECISIONS, MAX_LENGTH_IDX, MatchedRows, parse_date_if_exists
from ._to_match import SIMPLE_DOCKET_WITHOUT_DATE, simple_two_letter_docket_category


def get_date(raw: str, match: Match) -> Optional[date]:
    """Remove the docket (e.g. GR 1231) from the docket string (e.g. GR 1231, Dec. 1, 2000), to get the date text; parse this into a date object."""
    if docket_serial_text := match.group("undated"):
        return parse_date_if_exists(raw.removeprefix(docket_serial_text))
    return None


def valid_idx(candidate: str) -> Union[NoReturn, str]:
    if isinstance(candidate, int):
        candidate = str(candidate)
        if len(candidate) >= MAX_LENGTH_IDX:
            raise Exception("Valid idx - presently less than 100 characters")
    return candidate


def is_docket(raw: str) -> Optional[dict]:
    """Get dict sourced from a compiled regex pattern, with the date if possible"""
    raw = raw.strip().lower()

    if not (m := SIMPLE_DOCKET_WITHOUT_DATE.match(raw)):
        return None

    if not (cat := simple_two_letter_docket_category(m.group("cat"))):
        return None
    res = {"docket_cat": cat}

    if not (idx := valid_idx(m.group("idx").upper())):
        return None
    res |= {"docket_idx": idx}

    if (dt := get_date(raw, m)) and (formatted := dt.strftime("%Y-%m-%d")):
        res |= {"docket_dated": formatted}

    return res


def rows_from_docket(
    validated_docket_category: str,
    validated_docket_idx: str,
    formatted_date_string: str = None,
) -> MatchedRows:
    """The `cat` and `idx` in the query string are fixed table fields;
    The `cat_param` and `idx_param` are sourced from values of the defined dict of `is_docket`
    The query string is constructed with validated parameter variables

    Args:
        validated_docket_category (str): Value is sourced `is_docket`; created by `get_docket_category`
        validated_docket_idx (str): Value is sourced `is_docket`; validated by `valid_idx`
        formatted_date_string (str, optional): Value is sourced `is_docket`; created by `get_date`. Defaults to None.

    Returns:
        MatchedRows: [description]
    """
    queries: list[str] = ["cat = :cat_param", "idx like :idx_param"]
    params: dict[str, str] = {
        "cat_param": validated_docket_category,
        "idx_param": f"%{validated_docket_idx}%",
    }
    if formatted_date_string:
        queries.append("date(date_prom) = date(:date_param)")
        params |= {"date_param": formatted_date_string}

    return MatchedRows(
        DECISIONS.count_where(" and ".join(queries), params),
        DECISIONS.rows_where(" and ".join(queries), params),
    )


def rows_fom_docket_text(raw: str) -> Optional[MatchedRows]:
    return rows_from_docket(*d.values()) if (d := is_docket(raw)) else None
