import re
from typing import Iterator, Optional, Pattern

from casedockets.ac_patterns import ac_key
from casedockets.am_patterns import am_key
from casedockets.bm_patterns import bm_key
from casedockets.gr_patterns import gr_key

from ..config import MAX_LENGTH_IDX


def clean_idx(raw: str) -> str:
    return raw[:MAX_LENGTH_IDX].strip()


def construct_docket_categories() -> Iterator[str]:
    for i in ("gr", "am", "ac", "bm"):
        yield rf"(?P<{i}>{i[0]}\s*\.?\s*{i[1]}\s*\.?)"


DOCKET_OPTIONS: str = "|".join(construct_docket_categories())
SIMPLE_DOCKET_WITHOUT_DATE: Pattern = re.compile(
    rf"""(?P<undated>
        (?P<cat>{DOCKET_OPTIONS})
        \s*
        (no
            s?
            \.?
        )? # number keyword
        \s*
        (?P<idx>[\w-]+) # dashed word character
        \s*
    )""",
    re.I | re.X,
)


def simple_two_letter_docket_category(raw: str) -> Optional[str]:
    """Return if possible either: 'gr', 'am', 'ac', 'bm'"""
    pattern = re.compile(DOCKET_OPTIONS, re.I)
    return res.lastgroup if (res := pattern.search(raw)) else None


def source_simple(raw: str) -> Optional[dict]:
    idx = " ".join(i for i in raw.split() if re.search(r"\d+|&|(and)", i))
    text = " ".join(i for i in raw.split() if not re.search(r"\d+|&|(and)", i))
    if not (cat := simple_two_letter_docket_category(text)):
        return None

    return {"cat": cat, "idx": clean_idx(idx)}


def get_cat_idx_from_orig(
    raw: str, simple_category: str, regex_string: str
) -> Optional[dict]:
    if not (match := re.compile(regex_string, re.I | re.X).search(raw)):
        return None
    match_string = match.group()
    culled = raw.removeprefix(match_string).strip(" *")
    if culled.startswith("No."):
        culled = culled.removeprefix("No.")
    elif culled.startswith("NO."):
        culled = culled.removeprefix("No.")
    return {"cat": simple_category, "idx": clean_idx(culled)}


def source_orig(raw: str) -> Optional[dict]:
    return (
        get_cat_idx_from_orig(raw, "am", am_key)
        or get_cat_idx_from_orig(raw, "ac", ac_key)
        or get_cat_idx_from_orig(raw, "bm", bm_key)
        or get_cat_idx_from_orig(raw, "gr", gr_key)
    )


def get_cat_idx_from_docket(
    raw: str, simple_category: str, text_to_remove: str
) -> Optional[dict]:
    if not text_to_remove in raw:
        return None
    culled = raw.removeprefix(text_to_remove)
    separated = culled.split(",")
    return {"cat": simple_category, "idx": clean_idx(separated[0])}


def source_docket(raw: str) -> Optional[dict]:
    return (
        get_cat_idx_from_docket(raw, "am", "Administrative Matter")
        or get_cat_idx_from_docket(raw, "ac", "Administrative Case")
        or get_cat_idx_from_docket(raw, "bm", "Bar Matter")
        or get_cat_idx_from_docket(raw, "gr", "General Register")
    )


def updated_cat_idx(d: dict) -> dict:
    if d.get("orig_idx", None):
        if res := source_simple(d["orig_idx"]):
            return res
        if res := source_orig(d["orig_idx"]):
            return res
    if d.get("docket", None):
        if res := source_docket(d["docket"]):
            return res
    return {}
